% FD1d
% Arendt, Urban: Partielle Differenzialgleichungen, 2. Aufl. 2018
%
%  Finite Differenzen Verfahren fuer das RWP der GDL
%      -d^2u/dx^2 = f(x), 0 < x < 1,
%       u(0) = 0,
%       u(1) = 0.
%  
% Naeherungsloesungen auf einer Gitterfolge
% Angabe der exakten Loesung und Fehlerberechnung
% log-log-Plot des Fehlers ueber der Gitterweite
%  
% Berechnet die Daten fuer Abbildung 9.2

close all
clear all
format long e

    alpha = 0.5;

    nlevels=14;
    nvec = zeros(nlevels,1);
    fprintf('dof\t sin\t alpha\n');
    for k = 1:nlevels
        N = 2 * 2^k - 1;       %  Anzahl der inneren Punkte
        nvec(k) = N;           %  
        h = 1/(N+1);           %  Gitterweite h
        x = [0:h:1]';          %  Gitterpunkte inkl. Randpunkte
        x_interior = x(2:N+1); %  Innere Gitterpunkte
  
        % Aufstellen der Systemmatrix A_h der Dimension NxN
        % Speichern im Matlab sparse-Format
  
        Ah_diag = 2 * ones(N,1);
        Ah_sub = -1 * ones(N-1,1);
        A_h = spdiags([[Ah_sub;0],Ah_diag,[0;Ah_sub]],[-1 0 1],N,N);

        % rechte Seite b_h / zunaechst an den inneren Punkten
        %                    dann an den Randpunkten
        rhs_function_sin = rhs_sin(x_interior);
        rhs_function_alpha = rhs_alpha(x_interior,alpha);
    
        % exakte Loesung u_exact
        u_exact_sin = sol_sin(x);
        u_exact_alpha = sol_alpha(x,alpha);
        
        u_left  = 0;  %  Left BC
        u_right = 0;  %  Right BC
        b_h_sin = h^2 * rhs_function_sin;
        b_h_alpha = h^2 * rhs_function_alpha;
        b_h_sin(1) = b_h_sin(1) + u_left;
        b_h_sin(N) = b_h_sin(N) + u_right;
        b_h_alpha(1) = b_h_alpha(1) + u_left;
        b_h_alpha(N) = b_h_alpha(N) + u_right;
    
        % Loese LGS A_h*uh_interior = b_h 
        % erweitere Vektor um RBen
    
        uh_interior = A_h\b_h_sin;
        uh_sin = [u_left; uh_interior; u_right];
        uh_interior = A_h\b_h_alpha;
        uh_alpha = [u_left; uh_interior; u_right];
    
        % diskrete inf-Norm des Fehlers
        Eh_sin   = norm(uh_sin - u_exact_sin,'inf');
        Eh_alpha = norm(uh_alpha - u_exact_alpha,'inf');

        fprintf('%i\t %15.13e\t %15.13e\n',N,Eh_sin,Eh_alpha);
    
        %  Fehler und Gitterweite speichern
        Ehvec_sin(k)   = Eh_sin;
        Ehvec_alpha(k) = Eh_alpha;
        hvec(k) = h;
    end

    % Fehlerplot
    figure(1)
    subplot(2,1,1), plot(x,u_exact_sin-uh_sin,'-');
    title({'Fehler bzgl. $u_{\sin}$'},'Interpreter','latex')
    subplot(2,1,2), plot(x,u_exact_alpha-uh_alpha,'-');
    title({'Fehler bzgl. $u_{\alpha}$'},'Interpreter','latex')

    %  Plot Fehler Eh vs. h in log-log-Skala
    figure(2)
    loglog(nvec,Ehvec_sin,'k-o',nvec,Ehvec_alpha,'r-*');
    xlabel('d.o.f. n');
    ylabel('E_h = max |u_h - u_{exact}|');
    title('Log-log plot des Fehlers E_h bzgl. dof N');
    leg1 = legend('$u_{\sin}$','$u_\alpha$');
    set(leg1,'Interpreter','latex');
    set(leg1,'FontSize',17);    
    
    %  Gemittelte Konvergenzrate
    conv_rate_sin   = mean( diff(log(Ehvec_sin)) ./ diff(log(hvec)) );
    conv_rate_alpha = mean( diff(log(Ehvec_alpha)) ./ diff(log(hvec)) );
    fprintf('Rate h^p, mit p = %4.3e bzw. %4.3e\n',conv_rate_sin,conv_rate_alpha);
    