% FE1d
% Arendt, Urban: Partielle Differenzialgleichungen, 2. Aufl. 2018
%
%  Finite Elemente Verfahren fuer das RWP der GDL
%      -d^2u/dx^2 = f(x), 0 < x < 1,
%       u(0) = 0,
%       u(1) = 0.
%  
% Naeherungsloesungen auf einer Gitterfolge
% Angabe der exakten Loesung und Fehlerberechnung
% log-log-Plot des Fehlers ueber der Gitterweite
%  
% Berechnet die Daten fuer Abbildung 9.13

    alpha = 0.5;

    diffusion_coeff = @(x) 1+0.0*x;

    true_sin   = @(x) sol_sin(x);           % u_sin
    rhsf_sin   = @(x) rhs_sin(x);           % f_sin
    true_alpha = @(x) sol_alpha(x,alpha);   % u_alpha
    rhsf_alpha = @(x) rhs_alpha(x,alpha);   % f_alpha

    nlevels = 14; 
    nvec = zeros(nlevels,1);
    Eh_vec = zeros(nlevels,1);
    hvec = zeros(nlevels,1);
 
    fprintf('dof\t erralpha\t errsin\n');

    
    for k = 1:nlevels

        %  Setup FE-Gitter fuer Level k 
        n = 4 * 2^k - 1;                   %  # innerer Gitterpunkte
        nvec(k) = n;
        h = 1/(n+1);                       %  Gitterweite
        x = [0:h:1]';                      %  Giterpunkte
        xfine = [0:h/8:1]';                %  feines Gitter fuer Lsg
        x_int = x(2:n+1);                  %  innere Knoten
        x_mid = [h/2; x_int+h/2];          %  Element-Mittelpunkte

        %  Werte Kraft f(x) und Diffusionskoeffizienten kappa
        %  an den Mittelpunkten aus. Setze RBen

        f_mid_sin   = rhsf_sin(x_mid);
        f_mid_alpha = rhsf_alpha(x_mid);
        kappa_mid   = diffusion_coeff(x_mid);  
        u_left  = 0;
        u_right = 0;

        %  Berechne FEM rhs b mit Mittelpunktregel 
        %  haenge RBen an
        b_sin = h/2 * (f_mid_sin(1:n) + f_mid_sin(2:n+1));
        b_sin(1) = b_sin(1) + u_left / h;          
        b_sin(n) = b_sin(n) + u_right / h;
    
        b_alpha = h/2 * (f_mid_alpha(1:n) + f_mid_alpha(2:n+1));
        b_alpha(1) = b_alpha(1) + u_left / h;          
        b_alpha(n) = b_alpha(n) + u_right / h;

        
        %  FEM Steifigkitsmatrix A.
        Adiag = (kappa_mid(1:n) + kappa_mid(2:n+1)) / h;
        Asub = -kappa_mid(2:n) / h;
        Asuper = Asub;

        %  Matlab sparse-Matrix-Format
        A = spdiags([[Asub;0],Adiag,[0;Asub]],[-1 0 1],n,n);
       
        %  Loese und haenge RBen an 
        u_interior = A \ b_sin;
        u_h_sin = [u_left; u_interior; u_right];
        
        u_interior = A \ b_alpha;
        u_h_alpha = [u_left; u_interior; u_right];
  
        %  Max-Fehler an Gitterpunkten
        hvec(k) = h;
    
        %  Auswertung an feinerem Gitter
        u_true_sin    = true_sin(xfine);  
        uh_fine_sin   = interp1q(x,u_h_sin,xfine);
        u_true_alpha  = true_alpha(xfine);  
        uh_fine_alpha = interp1q(x,u_h_alpha,xfine);
    
        Eh_vec_sin(k)   = sqrt(h)*norm(uh_fine_sin - u_true_sin,2);
        Eh_vec_alpha(k) = sqrt(h)*norm(uh_fine_alpha - u_true_alpha,2);

    
        fprintf('%15.12e\t %15.12e\t %15.12e\n',n,Eh_vec_alpha(k),Eh_vec_sin(k));
 
  end

  %  Plot error vector E_h vs. h on a log-log scale.

    loglog(nvec,Eh_vec_sin,'k-o',nvec,Eh_vec_alpha,'r-*');
    xlabel('d.o.f. n');
    ylabel('E_h = max |u_h - u_{exact}|');
    title('Log-log plot des Fehlers E_h bzgl. dof N');
    leg1 = legend('$u_{\sin}$','$u_\alpha$');
    set(leg1,'Interpreter','latex');
    set(leg1,'FontSize',17);    
    hold off;
  
    %  Compute rate of convergence.
    conv_rate_sin   = mean( diff(log(Eh_vec_sin)) ./ diff(log(hvec)) );
    conv_rate_alpha = mean( diff(log(Eh_vec_alpha)) ./ diff(log(hvec)) );
    fprintf('Rate h^p, mit p = %4.3e bzw. %4.3e\n',conv_rate_sin,conv_rate_alpha);
    