% Poisson-Problem auf einem Polygon
close all
clear all

model = createpde;
R1 = [3,5,-1,1,0.8,0,-0.8,...
            -1,-1,0.5,1,0.5]';

gm = [R1];
sf = 'R1';


ns = char('R1');
ns = ns';
g = decsg(gm,sf,ns);

geometryFromEdges(model,g);
figure(1)
pdegplot(model,'EdgeLabels','on')
axis equal
xlim([-1.1,1.1]);
ylim([-1.1,1.1]);


applyBoundaryCondition(model,'dirichlet','Edge',...
    1:model.Geometry.NumEdges,'u',0);

specifyCoefficients(model,'m',0,...
                            'd',0,...
                            'c',1,...
                            'a',0,...
                            'f',1);

maxk=4;
H_exact=1/2^(maxk+1);
                        
finemesh = generateMesh(model,'Hmax',H_exact);
exact_results = solvepde(model);                    

v = linspace(-1,1,101);
[X,Y] = meshgrid(v);
querypoints = [X(:),Y(:)]';
u_exact = interpolateSolution(exact_results,querypoints);

uintrp = reshape(u_exact,size(X));
uintrp(isnan(uintrp))=0;

figure(2)
mesh(X,Y,uintrp);
colormap(gray);
%----------------------------------------------------
set(gca,'fontsize',20)
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = paperWidth*ratio;

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

print(gcf,'-dpdf','FE2D-B3-Sol.pdf');
%----------------------------------------------------

fprintf('k\t N\t L2Error\t H1Error\n');

 for k=1:maxk
    Hk = 1/2^k;
    hvec(k) = Hk;
    mesh_k  = generateMesh(model,'Hmax',Hk);
    Nvec(k) = size(mesh_k.Nodes,2);
    k_results = solvepde(model);                    
    u_k = interpolateSolution(k_results,querypoints);
    
    error_k=u_k-u_exact;   
    error_k(isnan(error_k))=0;
    
    l2error(k) = norm(error_k,2);
    h1error(k) = norm(diff(error_k),2);
    
    fprintf('%i\t %i\t %15.12e\t %15.12e\n',...
        k,Nvec(k),...
        l2error(k),h1error(k));
 end

figure(3)
p=pdeplot(model,'ColorMap','gray');
p(1).Color='black';
p(2).Color='black';
%----------------------------------------------------
set(gca,'fontsize',20)
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = abs(paperWidth*ratio);

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

print(gcf,'-dpdf','FE2D-B3-Mesh.pdf');
%----------------------------------------------------

 
%  Gemittelte Konvergenzrate
conv_rate_l2 = mean( diff(log(l2error)) ./ diff(log(hvec)) );
conv_rate_h1 = mean( diff(log(h1error)) ./ diff(log(hvec)) );
fprintf('Rate h^p, mit p = %4.3e bzw. %4.3e\n',...
                    conv_rate_l2,conv_rate_h1);

