% Waermeleitungsgleichung mit Dirichlet-RBen
% - Crank-Nicolson-Verfahren und FD -
%
% x=0 : Dirichlet, homogen
% t=0 : Anfangsbedingung (=u0!)
%
% h : Ortsschrittweite
% k : Zeitschrittweite
% n0 : Anzahl der Iterationen*10
%
%

% Parameter
M=100;    % Anzahl Ortsschritte
N=100;  % Anzahl Zeitschritte
T=1;

% Konstanten
dx=1/M
dt=T/N


%f  = @(t,x) 0.0;          % rechte Seite
f  = @(t,x) 2.*(1-t)-x.*(1.-x); % rechte Seite
u0 = @(x)   x.*(1.-x);    % Anfangsbedingungen

x =[0:dx:1];             % Gitterpunkte
xi=[dx:dx:1-dx];           % innere Gitterounkte
n=length(x);            % Gitterpunkte
Ui = u0(xi);            % Anfangsbedingung
Ui = Ui';

[xg,yg]=meshgrid([0:dx:1],[0:dt:T]);

%-- Aufstellung der Steifigkeitsmatrix
lambda=dt/(dx^2) % CFL-Zahl

%-- Systemmatrix fuer CN
a=zeros(n-2,1);
b=zeros(n-2,1);
c=zeros(n-2,1);

a(1:n-2)=1+lambda;
b(2:n-2)=-lambda/2;
c(1:n-3)=-lambda/2;
B=spdiags([c a b], -1:1, n-2, n-2);


a(1:n-2)=1-lambda;
b(2:n-2)=lambda/2;
c(1:n-3)=lambda/2;
A=spdiags([c a b], -1:1, n-2, n-2);


cpe=cputime;
l=0;
zg = zeros(N+1,n);
zg(1,2:n-1)=Ui;
zg(1,1)=0;
zg(1,n)=0;

for i=1:N
    tk=(i+0.5)*dt; 
    fk = f(tk,xi);
        
    d = A*Ui+dt*fk';

    Ui=B\d;
 
    zg(i+1,2:n-1)=Ui';
    zg(i+1,1)=0;
    zg(i+1,n)=0;
end;
ept=cputime-cpe


mesh(xg,yg,zg); 
view(135,30);
colormap(gray);
%xlabel('x');ylabel('t');zlabel('U(t,x)');
%----------------------------------------------------
set(gca,'fontsize',20)
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = abs(paperWidth*ratio);

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

print(gcf,'-dpdf','WLG-FD-CrankNicolson.pdf');
%----------------------------------------------------
