%   Numerisches Beispiel mit MATLAB zum expliziten Eulerverfahren fuer
%   u_t = u_xx                  in (0,T)x(0,1),
%   u(t,0)=u(t,1)=0             (0<t<T),
%   u(0,x)=u_0(x)               (0<x<1).
clear;

% Parameter
M=30;    % Anzahl Ortsschritte
%N=1780;  % Anzahl Zeitschritte
N=1800;  % Anzahl Zeitschritte
T=1;

% Anfangsdaten
u0 = @(x) x.*(1-x);

% Konstanten
dx=1/M
dt=T/N
nu=dt/dx^2
grid=linspace(0,1,M+1);

% Initialisiere Tridiagonalmatrix A  mit
% A = (nu, 1 + 2 nu, nu) 
A = spdiags([     nu*ones(M-1,1), ...
            (1-2*nu)*ones(M-1,1), ...
                 nu*ones(M-1,1)],[-1,0,1],M-1,M-1);

% Anfangsbedingungen
u=u0(grid);
U=zeros(N+1,M+1);
U=[u];

% Schleife Zeitschritte
for n=1:N
  u(1)=0;
  u(2:M) = A * U(n,2:M)';
  u(M+1)=0;
  U=[U;u];  
end;

% Graphik
[nr,nc]=size(U);
inc=floor(nr/21);
colormap(gray);
mesh(0:inc*dt:T,grid,U([1:inc:nr],:)');
view(45,30);
%title(' Numerische Approximation von u_t=u_xx mit Expl. Euler-Verfahren');
% xlabel('t');
% ylabel('x');
% zlabel('U(t,x)');
%----------------------------------------------------
set(gca,'fontsize',20)
ps = get(gcf, 'Position');
ratio = (ps(4)-ps(2)) / (ps(3)-ps(1));
paperWidth = 12;
paperHeight = abs(paperWidth*ratio);

set(gcf, 'paperunits', 'centimeters');
set(gcf, 'papersize', [paperWidth paperHeight]);
set(gcf, 'PaperPosition', [0    0   paperWidth paperHeight]);

%print(gcf,'-dpdf','WLG-FD-EulerExplizit-30-1780.pdf');
print(gcf,'-dpdf','WLG-FD-EulerExplizit-30-1800.pdf');
%----------------------------------------------------

