close all;
clear all;

format long e;
Mmin = 10;
Mmax = 300;
n = Mmax-Mmin+1;

fileID = fopen('WLG_KonvHist.txt','w');
fprintf(fileID,'M\t ErrCN\t CpuCN\t ErrIE\t CpuIE\n');
for i=1:n
    [errsCN(i),eptCN(i)]=ErrorCN(Mmin+i-1);
    [errsIE(i),eptIE(i)]=ErrorIE(Mmin+i-1);
    M(i) = Mmin+i-1;
    fprintf(fileID,'%i\t %15.12e \t %15.12e\t %15.12e \t %15.12e\n',...
        M(i),errsCN(i),eptCN(i),errsIE(i),eptIE(i));
end
fclose(fileID);

loglog(M,errsCN,'k-',M,errsIE,'r--');
legend('Crank-Nicolson','Euler Implizit');
axis tight

function rhs = f(t,x)
%    rhs = 2.*(1-t)-x.*(1.-x); % rechte Seite
   rhs = 2.*(1-t).^2-2.*(1.-t).*x.*(1.-x); % rechte Seite
end

function ic = u0(x)
    ic = x.*(1.-x); % Anfangsbedingungen
%     ic = zeros(size(x)); % Anfangsbedingungen
end

function [errCN,ept] = ErrorCN(M)
    N=M;  % Anzahl Zeitschritte
    T=1.;

    % Konstanten
    dx=1./M;
    dt=T/N;

%     f  = @(t,x) 2.*(1-t)-x.*(1.-x); % rechte Seite
%     u0 = @(x)   x.*(1.-x);          % Anfangsbedingungen

    x =[0:dx:1];                % Gitterpunkte
    xi=[dx:dx:1-dx];            % innere Gitterounkte
    n=length(x);                % Gitterpunkte
    Ui = u0(xi);                % Anfangsbedingung
    Ui = Ui';

    [xg,yg]=meshgrid([0:dx:1],[0:dt:T]);

    %-- Aufstellung der Steifigkeitsmatrix
    lambda=dt/(dx^2); % CFL-Zahl

    %-- Systemmatrix fuer CN
    a=zeros(n-2,1);
    b=zeros(n-2,1);
    c=zeros(n-2,1);

    a(1:n-2)=1+lambda;
    b(2:n-2)=-lambda/2;
    c(1:n-3)=-lambda/2;
    B=spdiags([c a b], -1:1, n-2, n-2);

    a(1:n-2)=1-lambda;
    b(2:n-2)=lambda/2;
    c(1:n-3)=lambda/2;
    A=spdiags([c a b], -1:1, n-2, n-2);

    cpe=cputime;
    l=0;
    zg = zeros(N+1,n);
    zg(1,2:n-1)=Ui;
    zg(1,1)=0;
    zg(1,n)=0;

    for i=1:N
        tk=(i+0.5)*dt; 
        fk = f(tk,xi);
        
        d = A*Ui+dt*fk';

        Ui=B\d;
        zg(i+1,2:n-1)=Ui';
        zg(i+1,1)=0;
        zg(i+1,n)=0;
    end;
    ept=cputime-cpe;
    errCN=norm(zg(N+1,:),Inf);   
end


function [errIE,ept] = ErrorIE(M)
    N=M;  % Anzahl Zeitschritte
    T=1;

    % Konstanten
    dx=1/M;
    dt=T/N;

%     f  = @(t,x) 2.*(1-t)-x.*(1.-x); % rechte Seite
%     u0 = @(x)   x.*(1.-x);    % Anfangsbedingungen

    x =[0:dx:1];                % Gitterpunkte
    xi=[dx:dx:1-dx];            % innere Gitterounkte
    n=length(x);                % Gitterpunkte
    Ui = u0(xi);                % Anfangsbedingung
    Ui = Ui';

    [xg,yg]=meshgrid([0:dx:1],[0:dt:T]);

    %-- Aufstellung der Steifigkeitsmatrix
    lambda=dt/(dx^2); % CFL-Zahl

    %-- Systemmatrix fuer impliziten Euler
    a=zeros(n-2,1);
    b=zeros(n-2,1);
    c=zeros(n-2,1);

    a(1:n-2)=1+2*lambda;
    b(2:n-2)=-lambda;
    c(1:n-3)=-lambda;

    B=spdiags([c a b], -1:1, n-2, n-2);

    cpe=cputime;
    l=0;
    zg = zeros(N+1,n);
    zg(1,2:n-1)=Ui;
    zg(1,1)=0;
    zg(1,n)=0;

    for i=1:N
        tk=i*dt;
        fk = f(tk,xi);

        d = Ui-dt*fk';

        Ui=B\d;
        zg(i+1,2:n-1)=Ui';
        zg(i+1,1)=0;
        zg(i+1,n)=0;
    end;

    ept=cputime-cpe;
    errIE=norm(zg(N+1,:),Inf);   
end

