% Wave_FE2dLShape
% Arendt, Urban: Partielle Differenzialgleichungen, 2. Aufl. 2018
close all;
clear all;

numberOfPDE = 1;
model = createpde(numberOfPDE);
geometryFromEdges(model,@squareg);

figure(1)
pdegplot(model,'EdgeLabels','on');
ylim([-1.1 1.1]);
axis equal
title 'Geometry With Edge Labels Displayed';
xlabel x
ylabel y


% Koeffizienten fuer die Wellengleichung
specifyCoefficients(model,'m',1,...
                            'd',0,...
                            'c',1,...
                            'a',0,...
                            'f',0);

applyBoundaryCondition(model,'dirichlet','Edge',[2,4],'u',0);
applyBoundaryCondition(model,'neumann','Edge',([1 3]),'g',0);

u0 = @(location) atan(cos(pi/2*location.x));
ut0 = @(location) 3*sin(pi*location.x).*exp(sin(pi/2*location.y));
setInitialConditions(model,u0,ut0);

maxk=4;
H_exact=1/2^(maxk+1);
finemesh = generateMesh(model,'Hmax',H_exact);

n = 31;
tlist = linspace(0,5,n);

model.SolverOptions.ReportStatistics ='on';
result = solvepde(model,tlist);
u = result.NodalSolution;


figure
umax = max(max(u));
umin = min(min(u));
for i = 1:n
    pdeplot(model,'XYData',u(:,i),'ZData',u(:,i),'ZStyle','continuous',...
                  'Mesh','off','XYGrid','on','ColorBar','off');
    axis([-1 1 -1 1 umin umax]);
    caxis([umin umax]);
    xlabel x
    ylabel y
    zlabel u
    M(i) = getframe;
end

