% solves stationary Poisson problem
%   -Du = f, in Omega
%   u = g,   on Boundary
% 
% [X,Y] : mesh in a box containing computational domain Omega
%   (hx and hy are assumed to be equal)
% G : contains indeces in [X,Y] defining the computational domain Omega
%   (0 if point does not belong to Omega)
% f : function for evaluating rhs of Poisson problem
% bnd : struct array defining the boundary of Omega
%    .g : function to evaluate on this boundary
%    .p : indices of points into [X,Y] on this boundary
%    .t : type of boundary [1,2,3,4] for [top,right,bottom,left] boundary
function ud = solveStat(X,Y,f,bnd,G)

Np2 = size(G,2);
h = (max(X(:))-min(X(:)))/(Np2-1);

% interior points
iplog = ~ismember(G,vertcat(0,bnd.p));
ip = G(iplog);

% compute discrete Laplacian
%   temporary G for usage in delsq that needs different enumaration than G!
Gtmp = zeros(size(G));
Gtmp(iplog) = 1:numel(ip);
% use delsq
D = h^-2*delsq(Gtmp);

% initialize u
ud = zeros(Np2,Np2);
% set DBC in ud
for k=1:length(bnd)
  ud(bnd(k).p) = bnd(k).g(X(bnd(k).p),Y(bnd(k).p));
end
% set rhs fd
fd = zeros(Np2);
% ... for DBC
incr = [1 -Np2 -1 Np2];
incr = incr([bnd.t]);
for k=1:length(bnd)
  fd(bnd(k).p+incr(k)) = fd(bnd(k).p+incr(k))+h^-2*ud(bnd(k).p);
end
% ... for given f
fd = fd + f(X,Y);
fd = fd(ip(:));

% solve pde in the interior
ud(ip) = D\fd;
